@@ -1,5 +1,3 @@
-from __future__ import print_function
-
 import importlib
 import inspect
 import os
@@ -8,10 +6,11 @@ import pydoc
 import sys
 import textwrap
 import subprocess
+import shlex
 
 from pprint import pprint
 from cmd import Cmd
-
+from cloudmesh.cmd5.CloudmeshPlugin import CloudmeshPlugin
 from cloudmesh.common.Printer import Printer
 from cloudmesh.common.Shell import Shell
 from cloudmesh.common.StopWatch import StopWatch
@@ -30,47 +29,32 @@ from cloudmesh.shell.plugin import PluginManager
 from cloudmesh.common.variables import Variables
 from cloudmesh.common.debug import VERBOSE
 
-#import cloudmesh.plugin
+
+# import cloudmesh.plugin
 
 
 def iter_namespace(ns_pkg):
     return pkgutil.iter_modules(ns_pkg.__path__, ns_pkg.__name__ + ".")
 
 
-#cloudmesh_plugins = {
+# cloudmesh_plugins = {
 #    name: importlib.import_module(name)
 #    for finder, name, ispkg
 #    in iter_namespace(cloudmesh.plugin)
-#}
+# }
 
-def print_list(elements):
-    """
-    prints the element of a list
-    :param elements: the elements to be printed
-    """
-    for name in elements:
-        print("*", name)
 
 def import_class(cl):
     d = cl.rfind(".")
-    classname = cl[d+1:len(cl)]
+    classname = cl[d + 1:len(cl)]
     m = __import__(cl[0:d], globals(), locals(), [classname])
     return getattr(m, classname)
 
-def inheritors(klass):
-    subclasses = set()
-    work = [klass]
-    while work:
-        parent = work.pop()
-        for child in parent.__subclasses__():
-            if child not in subclasses:
-                subclasses.add(child)
-                work.append(child)
-    return subclasses
 
 def get_class(str):
     return getattr(sys.modules[__name__], str)
 
+
 class Plugin(object):
     """
     Some simple methods to manage dynamic namespace plugins for cloudmesh.
@@ -83,7 +67,8 @@ class Plugin(object):
         while work:
             parent = work.pop()
             for child in parent.__subclasses__():
-                if child not in subclasses and child.__name__ not in ["CMShell", "CommandProxyClass"]:
+                if child not in subclasses and child.__name__ not in ["CMShell",
+                                                                      "CommandProxyClass"]:
                     subclasses.add(child)
                     work.append(child)
         return subclasses
@@ -97,9 +82,9 @@ class Plugin(object):
         module_list = []
         package = cloudmesh
         for importer, modname, ispkg in pkgutil.walk_packages(
-                path=package.__path__,
-                prefix=package.__name__ + '.',
-                onerror=lambda x: None):
+            path=package.__path__,
+            prefix=package.__name__ + '.',
+            onerror=lambda x: None):
             module_list.append(modname)
         return module_list
 
@@ -114,7 +99,7 @@ class Plugin(object):
         commands = []
         for module_name in module_list:
             if module_name.startswith(
-                    'cloudmesh.') and '.command.' in module_name:
+                'cloudmesh.') and '.command.' in module_name:
                 commands.append(module_name)
             # elif module_name.startswith('cloudmesh.plugin.'):
             #    commands.append(module_name)
@@ -165,13 +150,16 @@ class Plugin(object):
         if commands is None:
             commands = [c.split('.')[-1] for c in cls.classes()]
 
+
         class_commands = [cls.class_name(c) for c in commands]
+        # Console.bullets(class_commands)
+
         commands = [getattr(importlib.import_module(mod), cls) for (mod, cls) in
                     (commands.rsplit(".", 1) for commands in class_commands)]
-
         return commands
 
 
+
 Plugin.load()
 
 PluginCommandClasses = type(
@@ -179,6 +167,12 @@ PluginCommandClasses = type(
     tuple(PluginCommand.__subclasses__()),
     {})
 
+Console.init()
+
+# SysPluginManager.find_PluginCommands_in_sys()
+# c = cloudmesh.shell.command.PluginCommand
+# a = SysPluginManager.inheritors(c)
+# pprint (a)
 
 class CMShell(Cmd, PluginCommandClasses):
     """
@@ -199,39 +193,39 @@ class CMShell(Cmd, PluginCommandClasses):
 
     @command
     def do_commands(self, args, arguments):
-       """
-       ::
+        """
+        ::
 
-         Usage:
-               commands
+          Usage:
+                commands
 
-       """
-       arguments = dotdict(arguments)
+        """
+        arguments = dotdict(arguments)
 
-       for command in Plugin.list():
-           print(command)
+        for command in Plugin.list():
+            print(command)
 
-       names = self.get_names()
-       print("\n".join(names))
+        names = self.get_names()
+        print("\n".join(names))
 
-       #sys.exit(1)
-       for name in names:
-           if name[:3] == 'do_':
-               print(name)
+        # sys.exit(1)
+        for name in names:
+            if name[:3] == 'do_':
+                print(name)
 
-       d = []
-       for e in dir(cloudmesh):
-           if not e.startswith("__") and str(e) not in ["etc", "DEBUG"]:
-               d.append(e)
-       print(d)
-       VERBOSE(inheritors(PluginCommand))
-       modules = Plugin.modules()
-       VERBOSE(modules)
-       for m in modules:
-           print (m)
-       for m in sys.modules:
-           if str(m).startswith("cloudmesh."):
-               print (m)
+        d = []
+        for e in dir(cloudmesh):
+            if not e.startswith("__") and str(e) not in ["etc", "DEBUG"]:
+                d.append(e)
+        print(d)
+        VERBOSE(PluginManager.inheritors(PluginCommand))
+        modules = Plugin.modules()
+        VERBOSE(modules)
+        for m in modules:
+            print(m)
+        for m in sys.modules:
+            if str(m).startswith("cloudmesh."):
+                print(m)
 
     def precmd(self, line):
         StopWatch.start("command")
@@ -243,8 +237,8 @@ class CMShell(Cmd, PluginCommandClasses):
         try:
             variable = Variables()
             if "timer" not in variable:
-                variable["timer"] = "off"
-            if variable["timer"].lower() in ['on', 'true']:
+                variable["timer"] = "False"
+            if str(variable["timer"].lower()) in ['on', 'true']:
                 print("Timer: {:.4f}s ({})".format(StopWatch.get("command"),
                                                    line.strip()))
             variable.close()
@@ -312,8 +306,8 @@ class CMShell(Cmd, PluginCommandClasses):
         # print comment lines, but do not execute them
         # -----------------------------
         if line.startswith('#') \
-                or line.startswith('//') \
-                or line.startswith('/*'):
+            or line.startswith('//') \
+            or line.startswith('/*'):
             print(line)
             return ""
 
@@ -387,6 +381,26 @@ class CMShell(Cmd, PluginCommandClasses):
 
         return ""
 
+    # noinspection PyUnusedLocal
+    @command
+    def do_py(self, args, arguments):
+        """
+        ::
+
+           Usage:
+                py PROGRAM...
+
+            Arguments:
+                PROGRAM...  the command to be executed
+
+           Description:
+                py PROGRAM...  executes the command
+
+        """
+        result = eval(args)
+        print(result)
+        return ""
+
     # noinspection PyUnusedLocal
     @command
     def do_shell(self, args, arguments):
@@ -403,8 +417,6 @@ class CMShell(Cmd, PluginCommandClasses):
                 shell COMMAND  executes the command 
 
         """
-        # print ("Executing>", args, "<", sep='')
-        # os.system(args)
         os.system(str(args))
         return ""
 
@@ -450,9 +462,17 @@ class CMShell(Cmd, PluginCommandClasses):
             Description:
                 List available commands with "help" or detailed help with
                 "help COMMAND".
-        """
 
-        if arg:
+                In addition to the commands it also allows to print the api.
+                The classes that help is available for can be listed with
+
+                    cms info
+        """
+        if arg and "cloudmesh." in arg:
+            r = eval (f"help('{arg}')")
+            print (r)
+            return ""
+        elif arg:
             try:
                 func = getattr(self, 'help_' + arg)
             except AttributeError:
@@ -522,32 +542,116 @@ class CMShell(Cmd, PluginCommandClasses):
         ::
 
           Usage:
-                info [path|commands|files|cloudmesh]
+            info path
+            info commands
+            info files [errors] [--output=FORMAT]
+            info cloudmesh
+            info errors [--trace]
+            info sys [KEYWORD] [--plugins] [--output=FORMAT]
+            info
+
+          Options:
+            --format=FORMAT  the format to print the versions in [default: table]
 
           Description:
-                info
-                    provides internal info about the shell and its packages
 
+            info
+                provides internal info about the shell and its packages
+
+            The format option only works for
+
+                cms info files
         """
+
         arguments = dotdict(arguments)
+        _format = arguments["--output"] or "table"
 
         module_list = Plugin.modules()
 
         if arguments.commands:
+
             commands = Plugin.classes()
-            print_list(commands)
+            Console.bullets(commands)
+
         elif arguments.path:
+
             path_list = cloudmesh.__path__
-            print_list(path_list)
-        elif arguments.files:
-            commands = Plugin.modules()
-            for command in commands:
+            Console.bullets(path_list)
+
+        elif arguments.sys:
+
+            keyword = arguments["KEYWORD"]
+
+
+            all_modules = sys.modules.keys()
+            modules = []
+            for module in all_modules:
+                if keyword:
+                    if keyword in module:
+                        modules.append(module)
+                else:
+                   modules.append(module)
+
+
+            if arguments["--plugins"]:
+                plugins = []
+                for module in modules:
+                    try:
+                        _class = PluginManager.get_class(module)
+                        filename = inspect.getfile(_class)
+                        content = readfile(filename)
+                        if "PluginCommand" in content:
+                            entry = {
+                                "module": module,
+                                "filename": filename
+                            }
+                            plugins.append(entry)
+                    except Exception as e:
+                        # Console.error(str(e))
+                        pass
+                print(Printer.write(plugins,
+                                    order=["module", "filename"],
+                                    max_width=256,
+                                    output=_format))
+            else:
+                print("\n".join(modules))
+
+
+
+
+        elif arguments.files or arguments.errors:
+
+            modules = Plugin.modules()
+            longets = 0
+            data = []
+            for module in modules:
                 try:
-                    r = inspect.getfile(command)
-                    print("*", type(command))
+                    _class = get_class(module)
+                    r = inspect.getfile(_class)
+                    data.append({
+                        "module": module,
+                        "directory": r
+                    })
                 except Exception as e:
-                    print(e)
+                    if arguments.errors:
+                        if not "got str" in str(e) and "__version__" not in module:
+                            if "module" not in str(e):
+                                print()
+                                Console.error(f"Problem for {module}")
+                                Console.error(str(e),
+                                          traceflag=arguments["--trace"])
+                                print()
+                            else:
+                                Console.error(str(e), traceflag=arguments["--trace"])
+
+            if arguments.files:
+                print(Printer.write(data,
+                                    order=["module", "directory"],
+                                    max_width=256,
+                                    output=_format))
+
         elif arguments.help:
+
             for name in module_list:
                 p = "cloudmesh." + name
                 help_string = p + " not found."
@@ -560,14 +664,13 @@ class CMShell(Cmd, PluginCommandClasses):
                 print(help_string)
 
         else:
-            print_list(module_list)
-
 
+            Console.bullets(module_list)
 
     def preloop(self):
         """adds the banner to the preloop"""
 
-        lines = textwrap.dedent(self.banner).split("\n")
+        lines = textwrap.dedent(self.banner).splitlines()
         for line in lines:
             # Console.cprint("BLUE", "", line)
             print(line)
@@ -666,7 +769,8 @@ class CMShell(Cmd, PluginCommandClasses):
 
           Usage:
             version pip [PACKAGE]
-            version [--format=FORMAT] [--check=CHECK]
+            version [--format=FORMAT] [--check=CHECK] [--number]
+
            
 
           Options:
@@ -742,9 +846,10 @@ class CMShell(Cmd, PluginCommandClasses):
             output = subprocess.check_output(('grep', "cloudmesh"),
                                              stdin=pipcheck.stdout).decode(
                 "utf-8")
-            pkglines = output.strip().split("\n")
+            pkglines = output.strip().splitlines()
 
             for pkgline in pkglines:
+                # print (pkgline)
                 if "==" in pkgline:
                     values = pkgline.split("==")
                     pkg = values[0]
@@ -756,7 +861,9 @@ class CMShell(Cmd, PluginCommandClasses):
                         location = i.__file__
 
                         try:
-                            vlocation = path_expand(os.path.join(os.path.dirname(location), "__version__.py"))
+                            vlocation = path_expand(
+                                os.path.join(os.path.dirname(location),
+                                             "__version__.py"))
                             v = readfile(vlocation).split('"')[1].strip()
                         except:
                             v = "not found"
@@ -768,7 +875,7 @@ class CMShell(Cmd, PluginCommandClasses):
                             "source": location,
                             "VERSION": v
                         }
-                elif "git" in pkgline:
+                elif "git" in pkgline and ("installer" not in pkgline) and ("pi_burn" not in pkgline):
 
                     pkgline = pkgline.replace(
                         "-e git+git@github.com:cloudmesh-community/", "")
@@ -783,12 +890,17 @@ class CMShell(Cmd, PluginCommandClasses):
                     #
                     pname = pkg.replace("cloudmesh_", "cloudmesh.")
 
-                    i = importlib.import_module(pname)
+                    try:
+                        i = importlib.import_module(pname)
+                    except:
+                        continue
 
                     location = i.__file__
 
                     try:
-                        vlocation = path_expand(os.path.join(os.path.dirname(location), "__version__.py"))
+                        vlocation = path_expand(
+                            os.path.join(os.path.dirname(location),
+                                         "__version__.py"))
                         v = readfile(vlocation).split('"')[1].strip()
                     except:
                         v = "not found"
@@ -843,17 +955,35 @@ class CMShell(Cmd, PluginCommandClasses):
         print (versions)
         '''
 
-        print(Printer.write(versions,
+        if arguments["--number"]:
+
+            data = []
+            for key,entry in versions.items():
+                try:
+                    d = {
+                        'name': entry['package'],
+                        'version': entry['VERSION']
+                    }
+                    data.append(d)
+                except:
+                    pass
+
+            print(Printer.write(data,
                             output=arguments["--format"],
-                            order=["name", "package", "VERSION", "version", "source"],
+                                order=["name", "version"],
+                                header=["Package", "Version"],
                             sort_keys="name"))
+            return ""
+        else:
+            print(Printer.write(versions,
+                                output=arguments["--format"],
+                                order=["name", "package", "VERSION", "version",
+                                       "source"],
+                                sort_keys="name"))
         if arguments["--check"] in ["True"]:
             Shell.check_python()
 
 
-
-
-
 # noinspection PyBroadException,PyUnusedLocal
 def main():
     """cms.
@@ -919,6 +1049,26 @@ def main():
     script = arguments["SCRIPT"]
     command = arguments["COMMAND"]
 
+    #
+    # trick for the removal of the ", ' in the set command
+    #
+    if sys.platform != 'win32':
+        if len(sys.argv) == 3:
+            if sys.argv[1] == 'set' and "=" in sys.argv[2]:
+                command = command.replace("\"", "")
+                command = command.replace("=", "='", 1)
+                command = command + "'"
+        if len(sys.argv) >= 4:
+            if sys.argv[1] == 'config' and sys.argv[2] == 'set' and "=" in \
+                sys.argv[3]:
+                command = command.replace("\"", "")
+                command = command.replace("=", "='", 1)
+                command = command + "'"
+
+    # import ctypes
+    # ctypes.windll.kernel32.GetCommandLineA.restype = ctypes.c_char_p
+    # print (ctypes.windll.kernel32.GetCommandLineA())
+
     # context = CloudmeshContext(
     #    interactive=interactive,
     #    debug=debug,
